"""
This does the full thing:  it views from the retina, through V1,
to pyramid detection, then sets a fact on.  It also has the planning in.
So, it does the commands that are activated.  This can be set in the
short version in runNoCallBack, or all can be tested with the RunCallBack
version.  It has Ian's latest planning module but no error checking.
"""

#imports for PyNN
from pyNN.utility import get_script_args, normalized_filename
from neo.io import PyNNTextIO

#imports for image reading
import numpy
from PIL import Image
#Other NEAL files and functions

from visionClass import visionArea
from GMA_Nest import *

INPUT_NEURONS_HEIGHT = 50
INPUT_NEURONS_WIDTH = 50

DELAY = 1.0
output_filename='cabot1_out.dat'
SIM_LENGTH = 300.0
#SIM_LENGTH = 2000.0 #For the full plan test

CA_HEIGHT = 10
CA_WIDTH = 10


#------set up object to fact connections----------
#------set up object to fact connections----------
def factTurnsFactOn ():
    synWeight = 1.7
    connector = []
    for cFrom in range (0,5):
        for cTo in range (0,5):
            connector = connector + [(cFrom,cTo,synWeight,DELAY)]
    return connector
    
#-------------Print out neuron by neuron the firing times
def print_spikes(of, label, pop):
	print "\n\n%s\n" % label
	of.write("\n%s\n" % label)
	outDat = pop.get_data()
	for seg in outDat.segments:
		print(seg.spiketrains)
		of.write("\n%s\n" % str(seg.spiketrains))

#------Image Stuff
def openImageFile():
    try: 
#pyramid        im = Image.open("ed_3screenshot.jpg")
#stalactite        im = Image.open("ed_screenshot826.jpg") #stalactite
        #im = Image.open("../data/lStal1.jpg")  
        #im = Image.open("../data/rStal1.jpg")  
        im = Image.open("../data/rPyr1.jpg")  
        #im = Image.open("../data/lPyr1.jpg")  
    except:
        print "fail to open image"

    im = im.crop((0,0,600,600))
    #print im.size

    #shrink it
    size = (INPUT_NEURONS_WIDTH,INPUT_NEURONS_HEIGHT)
    im.thumbnail(size)

    #get the pixels
    pixels = im.load()

    ##convert rgb to grey scale
    inputMatrix = numpy.zeros((50,50))
    for inpRow in range (0,50):
        for inpCol in range (0,50):
            rgb = pixels[inpCol,inpRow]
            for i in range (0,3):
                inputMatrix[inpCol,inpRow] += rgb[i]
                inputMatrix[inpCol,inpRow]/=3
        
    ##set result image array
    resultImage = numpy.zeros((50,50))
    THRESHOLD = 50
    resultImageArray = []
    for outRow in range (0,50):
        for outColumn in range (0,50):
            if inputMatrix[outColumn,outRow] < THRESHOLD :
                #print outRow, outColumn
                pixelOn = (outRow*INPUT_NEURONS_WIDTH)+outColumn
                resultImageArray = resultImageArray + [pixelOn]

    return resultImageArray

        
def runNoCallBack():
    #set up clamped input from image
    pulseVis = DCSource(amplitude=0.0038, start=0.0, stop=SIM_LENGTH)
    inputAssembly = Assembly(vision.getInputPopulation()[imagePixels])
    pulseVis.inject_into(inputAssembly)
    run(SIM_LENGTH)

#------------Main Body---------------
simulator_name = get_script_args(1)[0]  
exec("from pyNN.%s import *" % simulator_name)

setup(timestep=DELAY,min_delay=DELAY,max_delay=DELAY,debug=0)

#----------------create neurons
#input neurons
cInputNeurons = INPUT_NEURONS_HEIGHT * INPUT_NEURONS_WIDTH 
vision = visionArea()
vision.allocateVisionNeurons()

#create fact neurons (for vision)
fact_cell_params = {'a': 0.05, 'd': 0,'i_offset' : 2}
leftFactAPop = Population(5,Izhikevich(**fact_cell_params))
leftFactBPop = Population(5,Izhikevich(**fact_cell_params))
rightFactAPop = Population(5,Izhikevich(**fact_cell_params))
rightFactBPop = Population(5,Izhikevich(**fact_cell_params))
pyramidFactAPop = Population(5,Izhikevich(**fact_cell_params))
pyramidFactBPop = Population(5,Izhikevich(**fact_cell_params))
stalactiteFactAPop = Population(5,Izhikevich(**fact_cell_params))
stalactiteFactBPop = Population(5,Izhikevich(**fact_cell_params))


#---------setup connections
vision.makeVisionSynapses()

#setup fact connections
synWeight = 5.0
connect(leftFactAPop, leftFactBPop, weight=synWeight, delay=DELAY)
connect(leftFactBPop, leftFactAPop, weight=synWeight, delay=DELAY)
connect(rightFactAPop, rightFactBPop, weight=synWeight, delay=DELAY)
connect(rightFactBPop, rightFactAPop, weight=synWeight, delay=DELAY)
connect(pyramidFactAPop, pyramidFactBPop, weight=synWeight, delay=DELAY)
connect(pyramidFactBPop, pyramidFactAPop, weight=synWeight, delay=DELAY)
connect(stalactiteFactAPop, stalactiteFactBPop, weight=synWeight, delay=DELAY)
connect(stalactiteFactBPop, stalactiteFactAPop, weight=synWeight, delay=DELAY)


#-------connect objects to facts
synapseArray = factTurnsFactOn()
Projection(vision.getLeftPyramid(),leftFactAPop,FromListConnector(synapseArray),StaticSynapse())
Projection(vision.getLeftStalactite(),leftFactAPop,FromListConnector(synapseArray),StaticSynapse())
Projection(vision.getRightPyramid(),rightFactAPop,FromListConnector(synapseArray),StaticSynapse())
Projection(vision.getRightStalactite(),rightFactAPop,FromListConnector(synapseArray),StaticSynapse())
Projection(vision.getLeftPyramid(),pyramidFactAPop,FromListConnector(synapseArray),StaticSynapse())
Projection(vision.getRightPyramid(),pyramidFactAPop,FromListConnector(synapseArray),StaticSynapse())
Projection(vision.getLeftStalactite(),stalactiteFactAPop,FromListConnector(synapseArray),StaticSynapse())
Projection(vision.getRightStalactite(),stalactiteFactAPop,FromListConnector(synapseArray),StaticSynapse())

#---------plan neurons and connections
print "Plan Connections"
maes1 = Maes()
#maes1.injections(0) #injection off here, as fed by cabot1, test independently GMA
#maes1.injectionTest1(0) #injection off here, as fed by cabot1, test independently GMA
maes1.injectionTestC(0) 
maes1.network()
maes1.networkFacts()

maesFactLeft = maes1.getChrisFact("LEFT")
maesFactRight = maes1.getChrisFact("RIGHT")
maesFactPyramid = maes1.getChrisFact("PYR")
maesFactStalactite = maes1.getChrisFact("STAL")
#undone
connect(leftFactAPop, maesFactLeft, weight=4.0, delay=DELAY)
connect(rightFactAPop, maesFactRight, weight=4.0, delay=DELAY)
connect(pyramidFactAPop, maesFactPyramid, weight=4.0, delay=DELAY)
connect(stalactiteFactAPop, maesFactStalactite, weight=4.0, delay=DELAY)


#-------------------setup recording

for i in range(0,4):
	p = maes1.getAction(i).recordSpikes()

vision.setVisionRecording()

leftFactAPop.record(['spikes','v'])
rightFactAPop.record(['spikes','v'])
pyramidFactAPop.record(['spikes','v'])
stalactiteFactAPop.record(['spikes','v'])


#comment - open file
of = open(output_filename, 'w')

#print inpPop[0].get_parameters()

#open image file
imagePixels = openImageFile()
#print imagePixels

runNoCallBack()

#--------------print results

labels=('TR','TL','FW','BW')	
for i in range(0,4):
	print_spikes(of,labels[i], maes1.getAction(i).getOnePop() )

print "crh left fact"
outAss = Assembly(leftFactAPop[0,1])
outDat = outAss.get_data()
for seg in outDat.segments:
    print(seg.spiketrains)


print "crh right fact"
outAss = Assembly(rightFactAPop[0,1])
outDat = outAss.get_data()
for seg in outDat.segments:
    print(seg.spiketrains)

print "pyramid fact"
outAss = Assembly(pyramidFactAPop[0,1])
outDat = outAss.get_data()
for seg in outDat.segments:
    print(seg.spiketrains)

print "stalactite fact"
outAss = Assembly(stalactiteFactAPop[0,1])
outDat = outAss.get_data()
for seg in outDat.segments:
    print(seg.spiketrains)

#vision.printVisionNets()
end()

