"""
The vision class without the connections.
"""


#imports for PyNN
from pyNN.utility import get_script_args, normalized_filename
from neo.io import PyNNTextIO
from pyNN.nest import *

from retinaConnections import set3x3OnConnections,set3x3OffConnections
from retinaConnections import set6x6OnConnections, set6x6OffConnections
from retinaConnections import set9x9OnConnections,set9x9OffConnections

from v1Connections import set6x6OnToHEdgeBottomConnections, set6x6OffToHEdgeBottomConnections
from v1Connections import set9x9OffToHEdgeBottomConnections
from v1Connections import set6x6OnToHEdgeTopConnections
from v1Connections import set6x6OffToHEdgeTopConnections, set9x9OffToHEdgeTopConnections
from v1Connections import set6x6OnToSEdgeLeftConnections
from v1Connections import set3x3OnToSEdgeRightConnections, set3x3OffToSEdgeRightConnections
from v1Connections import set6x6OnToSEdgeRightConnections
from v1Connections import set3x3OnToBEdgeRightConnections, set3x3OffToBEdgeRightConnections
from v1Connections import set6x6OffToBEdgeRightConnections
from v1Connections import set3x3OnToBEdgeLeftConnections, set6x6OffToBEdgeLeftConnections
from v1Connections import set3x3OffToBEdgeLeftConnections
from v1Connections import set3x3OnToAndAngleConnections,set6x6OnToAndAngleConnections
from v1Connections import set9x9OnToAndAngleConnections
from v1Connections import set3x3OnToLessThanAngleConnections,set6x6OffToLessThanAngleConnections
from v1Connections import set9x9OnToLessThanAngleConnections
from v1Connections import set3x3OnToGreaterThanAngleConnections,set6x6OffToGreaterThanAngleConnections
from v1Connections import set9x9OnToGreaterThanAngleConnections
from v1Connections import set3x3OnToOrAngleConnections, set6x6OnToOrAngleConnections
from v1Connections import set6x6OffToOrAngleConnections

INPUT_NEURONS_HEIGHT = 50
INPUT_NEURONS_WIDTH = 50

DELAY = 1.0

cInputNeurons = INPUT_NEURONS_HEIGHT * INPUT_NEURONS_WIDTH 

class visionArea:

    def printPopulationFirings(self,receptorPop):
        totalNeurons = INPUT_NEURONS_HEIGHT*INPUT_NEURONS_WIDTH
        for neuronNum in range (0,totalNeurons-1):
            if (neuronNum < totalNeurons):
                outAssembly = Assembly(receptorPop[neuronNum,(neuronNum+1)])
            else: 
                outAssembly = Assembly(receptorPop[neuronNum,0])
            outDat = outAssembly.get_data()
            seg = outDat.segments[0]
            spikeTrains = seg.spiketrains
            if len(spikeTrains) > 0:
                times = spikeTrains[0].times
                if len(times) > 0:
                    print neuronNum, len(times),
                    for time in times:
                        base = time.base
                        print base,
                    print " "

    def printReceptors(self):
        print "3x3 On"
        self.printPopulationFirings(self.retina3x3OnPop)
        print "3x3 Off"
        self.printPopulationFirings(self.retina3x3OffPop)
        print "6x6 On"
        self.printPopulationFirings(self.retina6x6OnPop)
        print "6x6 Off"
        self.printPopulationFirings(self.retina6x6OffPop)
        print "9x9 On"
        self.printPopulationFirings(self.retina9x9OnPop) 
        print "9x9 Off"
        self.printPopulationFirings(self.retina9x9OffPop)

    def printV1(self):
        print "HEdgeBottom"
        self.printPopulationFirings(self.v1HEdgeBottomPop)
        print "HEdgeTop"
        self.printPopulationFirings(self.v1HEdgeTopPop)
        print "SEdgeRight"
        self.printPopulationFirings(self.v1SEdgeRightPop)
        print "SEdgeLeft"
        self.printPopulationFirings(self.v1SEdgeLeftPop)
        print "BEdgeLeft"
        self.printPopulationFirings(self.v1BEdgeLeftPop)
        print "BEdgeRight"
        self.printPopulationFirings(self.v1BEdgeRightPop)
        print "AndAngle"
        self.printPopulationFirings(self.v1AndAnglePop)
        print "OrAngle"
        self.printPopulationFirings(self.v1OrAnglePop)
        print "LessThanAngle"
        self.printPopulationFirings(self.v1LessThanAnglePop)
        print "GreaterThanAngle"
        self.printPopulationFirings(self.v1GreaterThanAnglePop)

    def printVisionNets(self):
        print "Long Print Vision Nets"
#        self.printPopulationFirings(self.inpPop)
#        self.printReceptors()
#        self.printV1()
#        print "pyramid object"
#        self.printPopulationFirings(self.pyramidPop)
#        print "stalactite object"
#        self.printPopulationFirings(self.stalactitePop)
        print "top Hedge"
        outAss = Assembly(self.v1HEdgeTopPop[1366,1433])
        outDat = outAss.get_data()
        for seg in outDat.segments:
            print(seg.spiketrains)
        print "pyramid Left object"
        outAss = Assembly(self.pyramidLeftAPop[0,1])
        outDat = outAss.get_data()
        for seg in outDat.segments:
            print(seg.spiketrains)
        print "pyramid Right object"
        outAss = Assembly(self.pyramidRightAPop[0,1])
        outDat = outAss.get_data()
        for seg in outDat.segments:
            print(seg.spiketrains)
        print "stalactite Left object"
        outAss = Assembly(self.stalactiteLeftAPop[0,1])
        outDat = outAss.get_data()
        for seg in outDat.segments:
            print(seg.spiketrains)
        print "stalactite Right object"
        outAss = Assembly(self.stalactiteRightAPop[0,1])
        outDat = outAss.get_data()
        for seg in outDat.segments:
            print(seg.spiketrains)


    #---create neurons
    def allocateNeurons(self):
         #default a .02, b .2, c -65, d 2.0, i_offset 0
         inp_cell_params = {'a' : 0.0, 'b' :0.2, 'c' : -65, 'd' : 0.0}
         self.inpPop = Population(cInputNeurons,Izhikevich(**inp_cell_params))
         self.inpPop.initialize(v=-58.0)

         #create retinal neurons off centre different than on centre
         retina_cell_params = {'a' : 0.2, 'b' :0.2, 'c' : -65, 'd' : 2.0}
         self.retina3x3OnPop = Population(cInputNeurons,Izhikevich(**retina_cell_params))
         self.retina6x6OnPop  = Population(cInputNeurons,Izhikevich(**retina_cell_params))
         self.retina9x9OnPop  = Population(cInputNeurons,Izhikevich(**retina_cell_params))

         retina_cell_params = {'a' : 0.2, 'b' :0.2, 'c' : -65, 'd' : 2.0,'i_offset':1.0}
         self.retina3x3OffPop = Population(cInputNeurons,Izhikevich(**retina_cell_params))
         self.retina9x9OffPop = Population(cInputNeurons,Izhikevich(**retina_cell_params))

         retina_cell_params = {'a' : 0.2, 'b' :0.2, 'c' : -65, 'd' : 2.0,'i_offset':2.0}
         self.retina6x6OffPop = Population(cInputNeurons,Izhikevich(**retina_cell_params))

    def allocateV1Neurons(self):
         #create V1 neurons
         V1_c_p = {'a' : 0.2, 'b' :0.2, 'c' : -65, 'd' : 2.0}
         self.v1HEdgeBottomPop = Population(cInputNeurons,Izhikevich(**V1_c_p))
         self.v1SEdgeLeftPop  = Population(cInputNeurons,Izhikevich(**V1_c_p))
         self.v1BEdgeRightPop  = Population(cInputNeurons,Izhikevich(**V1_c_p))
         self.v1AndAnglePop  = Population(cInputNeurons,Izhikevich(**V1_c_p))
         self.v1LessThanAnglePop=Population(cInputNeurons,Izhikevich(**V1_c_p))
         self.v1GreaterThanAnglePop = Population(cInputNeurons,Izhikevich(**V1_c_p))
         self.v1HEdgeTopPop  = Population(cInputNeurons,Izhikevich(**V1_c_p))
         self.v1SEdgeRightPop  = Population(cInputNeurons,Izhikevich(**V1_c_p))
         self.v1BEdgeLeftPop  = Population(cInputNeurons,Izhikevich(**V1_c_p))
         self.v1OrAnglePop  = Population(cInputNeurons,Izhikevich(**V1_c_p))

    def allocateObjRecNeurons(self):
        #new pyramid objects 
        cell_params = {'a': 0.05,'d' : 0, 'i_offset' : 2}
        self.pyramidLeftAPop = Population(5,Izhikevich(**cell_params))
        self.pyramidLeftBPop = Population(5,Izhikevich(**cell_params))
        self.pyramidRightAPop = Population(5,Izhikevich(**cell_params))
        self.pyramidRightBPop = Population(5,Izhikevich(**cell_params))
        self.stalactiteLeftAPop = Population(5,Izhikevich(**cell_params))
        self.stalactiteLeftBPop = Population(5,Izhikevich(**cell_params))
        self.stalactiteRightAPop = Population(5,Izhikevich(**cell_params))
        self.stalactiteRightBPop = Population(5,Izhikevich(**cell_params))

    
#set up clamped input

    def allocateVisionNeurons(self):
        self.allocateNeurons()
        self.allocateV1Neurons()
        self.allocateObjRecNeurons()


    #----------------setup input to retina connections
    def makeInputToRetSynapses(self):
        print "Make Input to Retina Synapses"
        synapseArray = []
        #3x3On
        for row in range (1,49):
            for col in range (1,49):
                newSynapses = set3x3OnConnections(col,row)
                synapseArray = synapseArray + newSynapses
        Projection(self.inpPop,self.retina3x3OnPop,FromListConnector(synapseArray),StaticSynapse())

        #3x3Off
        synapseArray = []
        for row in range (1,49):
            for col in range (1,49):
                newSynapses = set3x3OffConnections(col,row)
                synapseArray = synapseArray + newSynapses
        Projection(self.inpPop,self.retina3x3OffPop,FromListConnector(synapseArray),StaticSynapse())

        #6x6On
        synapseArray = []
        for row in range (2,47):
            for col in range (2,47):
                newSynapses = set6x6OnConnections(col,row)
                synapseArray = synapseArray + newSynapses
        Projection(self.inpPop,self.retina6x6OnPop,FromListConnector(synapseArray),StaticSynapse())

        #6x6Off
        synapseArray = []
        for row in range (2,47):
            for col in range (2,47):
                newSynapses = set6x6OffConnections(col,row)
                synapseArray = synapseArray + newSynapses
        Projection(self.inpPop,self.retina6x6OffPop,FromListConnector(synapseArray),StaticSynapse())

        #9x9Off
        synapseArray = []
        for row in range (3,45):
            for col in range (3,45):
                newSynapses = set9x9OffConnections(col,row)
                synapseArray = synapseArray + newSynapses
        Projection(self.inpPop,self.retina9x9OffPop,FromListConnector(synapseArray),StaticSynapse())

        #9x9On
        synapseArray = []
        for row in range (3,45):
            for col in range (3,45):
                newSynapses = set9x9OnConnections(col,row)
                synapseArray = synapseArray + newSynapses
        Projection(self.inpPop,self.retina9x9OnPop,FromListConnector(synapseArray),StaticSynapse())

    #-------------------setup retina to V1 connections---
    def makeRetToV1Synapses(self):
        print "Make synapses from Ret to V1"
        #HedgeBottom
        synapseArray = []
        for row in range (0,INPUT_NEURONS_HEIGHT-1):
            for col in range (2,INPUT_NEURONS_HEIGHT-3):
                newSynapses = set6x6OnToHEdgeBottomConnections(col,row)
                synapseArray = synapseArray + newSynapses
        Projection(self.retina6x6OnPop,self.v1HEdgeBottomPop,FromListConnector(synapseArray),StaticSynapse())

        synapseArray = []
        for row in range (0,INPUT_NEURONS_HEIGHT-1):
            for col in range (2,INPUT_NEURONS_HEIGHT-3):
                newSynapses = set6x6OffToHEdgeBottomConnections(col,row)
                synapseArray = synapseArray + newSynapses
        Projection(self.retina6x6OffPop,self.v1HEdgeBottomPop,FromListConnector(synapseArray),StaticSynapse())

        synapseArray = []
        for row in range (0,INPUT_NEURONS_HEIGHT-1):
            for col in range (2,INPUT_NEURONS_HEIGHT-3):
                newSynapses = set9x9OffToHEdgeBottomConnections(col,row)
                synapseArray = synapseArray + newSynapses
        Projection(self.retina9x9OffPop,self.v1HEdgeBottomPop,FromListConnector(synapseArray),StaticSynapse())

        #SEdge Left Connections
        synapseArray = []
        for row in range (0,INPUT_NEURONS_HEIGHT):
            for col in range (0,INPUT_NEURONS_WIDTH):
                newSynapses = set6x6OnToSEdgeLeftConnections(col,row)
                synapseArray = synapseArray + newSynapses
#        Projection(self.retina6x6OnPop,self.v1SEdgeLeftPop,FromListConnector(synapseArray),StaticSynapse())
#        Projection(self.retina6x6OffPop,self.v1SEdgeLeftPop,FromListConnector(synapseArray),StaticSynapse())

        #rightBEdges
        synapseArray = []
        for row in range (0,INPUT_NEURONS_HEIGHT):
            for col in range (0,INPUT_NEURONS_WIDTH):
                newSynapses = set3x3OnToBEdgeRightConnections(col,row)
                synapseArray = synapseArray + newSynapses
        Projection(self.retina3x3OnPop,self.v1BEdgeRightPop,FromListConnector(synapseArray),StaticSynapse())
        
        synapseArray = []
        for row in range (0,INPUT_NEURONS_HEIGHT):
            for col in range (0,INPUT_NEURONS_WIDTH):
                newSynapses = set3x3OffToBEdgeRightConnections(col,row)
                synapseArray = synapseArray + newSynapses
        Projection(self.retina3x3OffPop,self.v1BEdgeRightPop,FromListConnector(synapseArray),StaticSynapse())

        synapseArray = []
        for row in range (0,INPUT_NEURONS_HEIGHT):
            for col in range (0,INPUT_NEURONS_WIDTH):
                newSynapses = set6x6OffToBEdgeRightConnections(col,row)
                synapseArray = synapseArray + newSynapses
        Projection(self.retina6x6OffPop,self.v1BEdgeRightPop,FromListConnector(synapseArray),StaticSynapse())

        #left BEdges
        synapseArray = []
        for row in range (0,INPUT_NEURONS_HEIGHT-1):
           for col in range (0,INPUT_NEURONS_HEIGHT):
               newSynapses = set3x3OnToBEdgeLeftConnections(col,row)
               synapseArray = synapseArray + newSynapses
        Projection(self.retina3x3OnPop,self.v1BEdgeLeftPop,FromListConnector(synapseArray),StaticSynapse())
        
        synapseArray = []
        for row in range (0,INPUT_NEURONS_HEIGHT-1):
           for col in range (0,INPUT_NEURONS_HEIGHT):
               newSynapses = set3x3OffToBEdgeLeftConnections(col,row)
               synapseArray = synapseArray + newSynapses
        Projection(self.retina3x3OffPop,self.v1BEdgeLeftPop,FromListConnector(synapseArray),StaticSynapse())

        synapseArray = []
        for row in range (0,INPUT_NEURONS_HEIGHT-1):
           for col in range (0,INPUT_NEURONS_HEIGHT):
               newSynapses = set6x6OffToBEdgeLeftConnections(col,row)
               synapseArray = synapseArray + newSynapses
        Projection(self.retina6x6OffPop,self.v1BEdgeLeftPop,FromListConnector(synapseArray),StaticSynapse())

        #andAngle
        synapseArray = []
        for row in range (0,INPUT_NEURONS_HEIGHT):
            for col in range (0,INPUT_NEURONS_WIDTH):
                newSynapses = set3x3OnToAndAngleConnections(col,row)
                synapseArray = synapseArray + newSynapses
        Projection(self.retina3x3OnPop,self.v1AndAnglePop,FromListConnector(synapseArray),StaticSynapse())

        synapseArray = []
        for row in range (0,INPUT_NEURONS_HEIGHT):
            for col in range (0,INPUT_NEURONS_WIDTH):
                newSynapses = set6x6OnToAndAngleConnections(col,row)
                synapseArray = synapseArray + newSynapses
        Projection(self.retina6x6OnPop,self.v1AndAnglePop,FromListConnector(synapseArray),StaticSynapse())

        synapseArray = []
        for row in range (0,INPUT_NEURONS_HEIGHT):
            for col in range (0,INPUT_NEURONS_WIDTH):
                newSynapses = set9x9OnToAndAngleConnections(col,row)
                synapseArray = synapseArray + newSynapses
        Projection(self.retina9x9OnPop,self.v1AndAnglePop,FromListConnector(synapseArray),StaticSynapse())

        #Less Than Angle
        synapseArray = []
        for row in range (0,INPUT_NEURONS_HEIGHT):
            for col in range (0,INPUT_NEURONS_WIDTH):
                newSynapses = set3x3OnToLessThanAngleConnections(col,row)
                synapseArray = synapseArray + newSynapses
        Projection(self.retina3x3OnPop,self.v1LessThanAnglePop,FromListConnector(synapseArray),StaticSynapse())

        synapseArray = []
        for row in range (0,INPUT_NEURONS_HEIGHT):
            for col in range (0,INPUT_NEURONS_WIDTH):
                newSynapses = set6x6OffToLessThanAngleConnections(col,row)
                synapseArray = synapseArray + newSynapses
        Projection(self.retina6x6OffPop,self.v1LessThanAnglePop,FromListConnector(synapseArray),StaticSynapse())
        
        synapseArray = []
        for row in range (0,INPUT_NEURONS_HEIGHT):
            for col in range (0,INPUT_NEURONS_WIDTH):
                newSynapses = set9x9OnToLessThanAngleConnections(col,row)
                synapseArray = synapseArray + newSynapses
        Projection(self.retina9x9OnPop,self.v1LessThanAnglePop,FromListConnector(synapseArray),StaticSynapse())

        #Greater Than Angle
        synapseArray = []
        for row in range (0,INPUT_NEURONS_HEIGHT):
            for col in range (0,INPUT_NEURONS_WIDTH):
                newSynapses = set3x3OnToGreaterThanAngleConnections(col,row)
                synapseArray = synapseArray + newSynapses
        Projection(self.retina3x3OnPop,self.v1GreaterThanAnglePop,FromListConnector(synapseArray),StaticSynapse())

        synapseArray = []
        for row in range (0,INPUT_NEURONS_HEIGHT):
            for col in range (0,INPUT_NEURONS_WIDTH):
                newSynapses = set6x6OffToGreaterThanAngleConnections(col,row)
                synapseArray = synapseArray + newSynapses
        Projection(self.retina6x6OffPop,self.v1GreaterThanAnglePop,FromListConnector(synapseArray),StaticSynapse())

        synapseArray = []
        for row in range (0,INPUT_NEURONS_HEIGHT):
            for col in range (0,INPUT_NEURONS_WIDTH):
                newSynapses = set9x9OnToGreaterThanAngleConnections(col,row)
                synapseArray = synapseArray + newSynapses
        Projection(self.retina9x9OnPop,self.v1GreaterThanAnglePop,FromListConnector(synapseArray),StaticSynapse())

        #topHEdges
        synapseArray = []
        for row in range (0,INPUT_NEURONS_HEIGHT-1):
            for col in range (0,INPUT_NEURONS_HEIGHT):
                newSynapses = set6x6OnToHEdgeTopConnections(col,row)
                synapseArray = synapseArray + newSynapses
        Projection(self.retina6x6OnPop,self.v1HEdgeTopPop,FromListConnector(synapseArray),StaticSynapse())

        synapseArray = []
        for row in range (0,INPUT_NEURONS_HEIGHT-1):
            for col in range (0,INPUT_NEURONS_HEIGHT):
                newSynapses = set6x6OffToHEdgeTopConnections(col,row)
                synapseArray = synapseArray + newSynapses
        Projection(self.retina6x6OffPop,self.v1HEdgeTopPop,FromListConnector(synapseArray),StaticSynapse())

        synapseArray = []
        for row in range (0,INPUT_NEURONS_HEIGHT-1):
            for col in range (0,INPUT_NEURONS_HEIGHT):
                newSynapses = set9x9OffToHEdgeTopConnections(col,row)
                synapseArray = synapseArray + newSynapses
        Projection(self.retina9x9OffPop,self.v1HEdgeTopPop,FromListConnector(synapseArray),StaticSynapse())

        #right SEdges
        synapseArray = []
        for row in range (0,INPUT_NEURONS_HEIGHT-1):
            for col in range (0,INPUT_NEURONS_HEIGHT):
                newSynapses = set3x3OnToSEdgeRightConnections(col,row)
                synapseArray = synapseArray + newSynapses
        Projection(self.retina3x3OnPop,self.v1SEdgeRightPop,FromListConnector(synapseArray),StaticSynapse())

        synapseArray = []
        for row in range (0,INPUT_NEURONS_HEIGHT-1):
            for col in range (0,INPUT_NEURONS_HEIGHT):
                newSynapses = set3x3OffToSEdgeRightConnections(col,row)
                synapseArray = synapseArray + newSynapses
        Projection(self.retina3x3OffPop,self.v1SEdgeRightPop,FromListConnector(synapseArray),StaticSynapse())

        synapseArray = []
        for row in range (0,INPUT_NEURONS_HEIGHT-1):
            for col in range (0,INPUT_NEURONS_HEIGHT):
                newSynapses = set6x6OnToSEdgeRightConnections(col,row)
                synapseArray = synapseArray + newSynapses
        Projection(self.retina6x6OnPop,self.v1SEdgeRightPop,FromListConnector(synapseArray),StaticSynapse())

        #or Angle
        synapseArray = []
        for row in range (0,INPUT_NEURONS_HEIGHT):
            for col in range (0,INPUT_NEURONS_WIDTH):
                newSynapses = set3x3OnToOrAngleConnections(col,row)
                synapseArray = synapseArray + newSynapses
        Projection(self.retina3x3OnPop,self.v1OrAnglePop,FromListConnector(synapseArray),StaticSynapse())
        
        synapseArray = []
        for row in range (0,INPUT_NEURONS_HEIGHT):
            for col in range (0,INPUT_NEURONS_WIDTH):
                newSynapses = set6x6OnToOrAngleConnections(col,row)
                synapseArray = synapseArray + newSynapses
        Projection(self.retina6x6OnPop,self.v1OrAnglePop,FromListConnector(synapseArray),StaticSynapse())
        
        synapseArray = []
        for row in range (0,INPUT_NEURONS_HEIGHT-1):
            for col in range (0,INPUT_NEURONS_HEIGHT):
                newSynapses = set6x6OffToOrAngleConnections(col,row)
                synapseArray = synapseArray + newSynapses
        Projection(self.retina6x6OffPop,self.v1OrAnglePop,FromListConnector(synapseArray),StaticSynapse())


    #-------------------setup Object Recognion and  V1 to ObjRec connections---
    #subproc
    def connectV1FeatureToObject (self, synWeight,leftStart, rightEnd):
        connector = []
        for cRow in range (0,INPUT_NEURONS_HEIGHT):
            for cCol in range (int(INPUT_NEURONS_HEIGHT*leftStart),int(INPUT_NEURONS_HEIGHT*rightEnd)):
                for toNeuron in range (0,5):
                    fromNeuron = cRow*INPUT_NEURONS_WIDTH + cCol
                    connector = connector + [(fromNeuron,toNeuron,synWeight,DELAY)]
        return connector

    #main proc
    def makeV1ToObjRecSynapses(self):
        print "Make V1 to Object Recognition Synapses"

        #Left
        #Big Weights
        #synapseArray = self.connectV1FeatureToObject(10.0,0,0.5)

        #Small Weights
        synapseArray = self.connectV1FeatureToObject(3.0,0,0.5)
        #Pyramid Left
        Projection(self.v1HEdgeBottomPop,self.pyramidLeftAPop,FromListConnector(synapseArray),StaticSynapse())
        Projection(self.v1BEdgeRightPop,self.pyramidLeftAPop,FromListConnector(synapseArray),StaticSynapse())
        #Projection(self.v1SEdgeLeftPop,self.pyramidLeftAPop,FromListConnector(synapseArray),StaticSynapse())
        Projection(self.v1AndAnglePop,self.pyramidLeftAPop,FromListConnector(synapseArray),StaticSynapse())
        
        #Stalactite Left
        Projection(self.v1HEdgeTopPop,self.stalactiteLeftAPop,FromListConnector(synapseArray),StaticSynapse())        
        Projection(self.v1SEdgeRightPop,self.stalactiteLeftAPop,FromListConnector(synapseArray),StaticSynapse())
        #Projection(self.v1BEdgeLeftPop,self.stalactiteLeftAPop,FromListConnector(synapseArray),StaticSynapse())
        Projection(self.v1OrAnglePop,self.stalactiteLeftAPop,FromListConnector(synapseArray),StaticSynapse())

        #Projection(self.v1LessThanAnglePop,self.stalactiteLeftAPop,FromListConnector(synapseArray),StaticSynapse())
        Projection(self.v1GreaterThanAnglePop,self.stalactiteLeftAPop,FromListConnector(synapseArray),StaticSynapse())



        #Right
        #Big Weights
        synapseArray = self.connectV1FeatureToObject(3.0,0.5,1)
        Projection(self.v1HEdgeTopPop,self.pyramidRightAPop,FromListConnector(synapseArray),StaticSynapse())
#        Projection(self.v1BEdgeRightPop,self.pyramidRightAPop,FromListConnector(synapseArray),StaticSynapse())

        #Small Weights
        synapseArray = self.connectV1FeatureToObject(2.0,0.5,1) 
        #Pyramid Right
        Projection(self.v1HEdgeBottomPop,self.pyramidRightAPop,FromListConnector(synapseArray),StaticSynapse())
        Projection(self.v1BEdgeRightPop,self.pyramidRightAPop,FromListConnector(synapseArray),StaticSynapse())
        #Projection(self.v1SEdgeLeftPop,self.pyramidRightAPop,FromListConnector(synapseArray),StaticSynapse())
        Projection(self.v1AndAnglePop,self.pyramidRightAPop,FromListConnector(synapseArray),StaticSynapse())
        
        #Stalactite Right
        Projection(self.v1HEdgeTopPop,self.stalactiteRightAPop,FromListConnector(synapseArray),StaticSynapse())        
        Projection(self.v1SEdgeRightPop,self.stalactiteRightAPop,FromListConnector(synapseArray),StaticSynapse())
        #Projection(self.v1BEdgeLeftPop,self.stalactiteRightAPop,FromListConnector(synapseArray),StaticSynapse())
        Projection(self.v1OrAnglePop,self.stalactiteRightAPop,FromListConnector(synapseArray),StaticSynapse())

#        Projection(self.v1LessThanAnglePop,self.pyramidRightAPop,FromListConnector(synapseArray),StaticSynapse())
#        Projection(self.v1GreaterThanAnglePop,self.pyramidRightAPop,FromListConnector(synapseArray),StaticSynapse())


    def makeObjRecSynapses(self):
        #new pyramid CA connections
        synWeight = 5.0
        connect(self.pyramidLeftAPop, self.pyramidLeftBPop,weight=synWeight, delay=DELAY)
        connect(self.pyramidLeftBPop, self.pyramidLeftAPop,weight=synWeight, delay=DELAY)
        connect(self.pyramidRightAPop, self.pyramidRightBPop,weight=synWeight, delay=DELAY)
        connect(self.pyramidRightBPop, self.pyramidRightAPop,weight=synWeight, delay=DELAY)
        connect(self.stalactiteLeftAPop,self.stalactiteLeftBPop,weight=synWeight,delay=DELAY)
        connect(self.stalactiteLeftBPop,self.stalactiteLeftAPop,weight=synWeight,delay=DELAY)
        connect(self.stalactiteRightAPop,self.stalactiteRightBPop,weight=synWeight,delay=DELAY)
        connect(self.stalactiteRightBPop,self.stalactiteRightAPop,weight=synWeight,delay=DELAY)

        #the vision object position facts are mutually exclusive
        inhibWeight = -3.0
        connect(self.pyramidLeftAPop,self.pyramidRightAPop,weight=inhibWeight,delay=DELAY)
        connect(self.pyramidLeftAPop,self.stalactiteLeftAPop,weight=inhibWeight,delay=DELAY)
        connect(self.pyramidLeftAPop,self.stalactiteRightAPop,weight=inhibWeight,delay=DELAY)
        connect(self.pyramidRightAPop,self.pyramidLeftAPop,weight=inhibWeight,delay=DELAY)
        connect(self.pyramidRightAPop,self.stalactiteLeftAPop,weight=inhibWeight,delay=DELAY)
        connect(self.pyramidRightAPop,self.stalactiteRightAPop,weight=inhibWeight,delay=DELAY)
        connect(self.stalactiteLeftAPop,self.pyramidLeftAPop,weight=inhibWeight,delay=DELAY)
        connect(self.stalactiteLeftAPop,self.pyramidRightAPop,weight=inhibWeight,delay=DELAY)
        connect(self.stalactiteLeftAPop,self.stalactiteRightAPop,weight=inhibWeight,delay=DELAY)
        connect(self.stalactiteRightAPop,self.pyramidLeftAPop,weight=inhibWeight,delay=DELAY)
        connect(self.stalactiteRightAPop,self.pyramidRightAPop,weight=inhibWeight,delay=DELAY)
        connect(self.stalactiteRightAPop,self.stalactiteLeftAPop,weight=inhibWeight,delay=DELAY)


    def makeVisionSynapses(self):
        self.makeV1ToObjRecSynapses()
        self.makeObjRecSynapses()
        self.makeInputToRetSynapses()
        self.makeRetToV1Synapses()


    def setVisionRecording(self):
        self.retina3x3OffPop.record(['spikes','v'])
        self.retina3x3OffPop.record(['spikes','v'])
        self.retina3x3OffPop.record(['spikes','v'])
        self.retina6x6OnPop.record(['spikes','v'])
        self.retina6x6OffPop.record(['spikes','v'])
        self.retina9x9OnPop.record(['spikes','v'])
        self.retina9x9OffPop.record(['spikes','v'])
        self.v1HEdgeBottomPop.record(['spikes','v'])
        self.v1HEdgeTopPop.record(['spikes','v'])
        self.v1SEdgeRightPop.record(['spikes','v'])
        self.v1SEdgeLeftPop.record(['spikes','v'])
        self.v1BEdgeLeftPop.record(['spikes','v'])
        self.v1BEdgeRightPop.record(['spikes','v'])
        self.v1AndAnglePop.record(['spikes','v'])
        self.v1LessThanAnglePop.record(['spikes','v'])
        self.v1GreaterThanAnglePop.record(['spikes','v'])
        self.pyramidLeftAPop.record(['spikes','v'])
        self.pyramidRightAPop.record(['spikes','v'])
        self.stalactiteLeftAPop.record(['spikes','v'])
        self.stalactiteRightAPop.record(['spikes','v'])



    def getInputPopulation(self):
        return self.inpPop

    def getLeftPyramid(self):
        return self.pyramidLeftAPop

    def getRightPyramid(self):
        return self.pyramidRightAPop

    def getLeftStalactite(self):
        return self.stalactiteLeftAPop

    def getRightStalactite(self):
        return self.stalactiteRightAPop


